/* exportDialog.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "exportDialog.h"
#include "exportingDialog.h"
#include "song.h"
#include "cd.h"
#include "queryResult.h"
#include "database.h"

#include "playlistProxy.h"

ExportDialog::
ExportDialog(const Dataset &data, 
	     const PlaylistProxy &playlist, int start_row,
	     bool all) :
  PalomaDialog(data),
  _playlist(playlist),
  _start_row(start_row),
  _symlink_button(NULL, "Make symbolic links"),
  _copy_button(_symlink_button, "Copy files"),
  _convert_button(_copy_button, "Convert files to: "),
  _root_label("Root directory:"),
  _filename_label("Filename pattern:"),
  _filename_explanation("Available variables in filename pattern:\n"
			"        $index - zero-based index within playlist\n"
			"        $basename - basename (without extension) of original file\n"
			"        $ext - filename extension of original file, includes dot\n"
			"        $name - song name\n"
			"        $performer, $writer\n"
			"        $cdtitle, $cdperformer, $cdwriter\n"
			"        $track - track number"),
  _max_time_check("Maximum total length of music to write (minutes):"),
  _ok_button("OK"),
  _cancel_button("Cancel")
{
  assert(_playlist.get_primary_table() == TI_songs ||
	 _playlist.get_primary_table() == TI_cds);

  // Get all of the soundfile definitions, except the CD-Rom definition.
  QueryResult qr = music_db->sql_command
    ("SELECT soundfile_id, description"
     " FROM soundfile, configure"
     " WHERE soundfile_id != configure.cdrom_type"
     " ORDER BY description");
  int num_rows = qr.get_num_rows();
  for (int r = 0; r < num_rows; r++) {
    int soundfile_id = qr.get_cell(r, 0).get_int();
    string description = qr.get_cell(r, 1).get_string();
    _convert_type.add_choice(soundfile_id, description);
  }
  _convert_type.finish_choices();


  // Get the first explanatory label.
  if (all) {
    if (_playlist.get_primary_table() == TI_songs) {
      _top_label.set_text
	("Export ALL songs in the database.  (Use the export utility\n"
	 "from a playlist dialog to export only some of the songs.)\n\n"
	 "This will export all songs, even those that are not part of\n"
	 "a CD.  A song that appears on multiple CD's will only appear\n"
	 "once in this export list.");
    } else {
      _top_label.set_text
	("Export ALL CD's in the database.  (Use the export utility\n"
	 "from a playlist dialog to export only some of the CD's.)\n\n"
	 "This will export all songs that appear on CD's.  Songs that do\n"
	 "not appear on any CD will not be a part of this export list.\n"
	 "A song that appears on multiple CD's will appear multiple times\n"
	 "in this export list.");
    }
  } else {
    string start_name;
    if (_playlist.get_primary_table() == TI_songs) {
      TableDataProxy song(TI_songs, _playlist.get_row(_start_row)->_primary_key);
      start_name = get_formatted_song_name(song);
    } else {
      TableDataProxy cd(TI_cds, _playlist.get_row(_start_row)->_primary_key);
      start_name = get_formatted_cd_name(cd);
    }
    
    _top_label.set_text("Export song files from playlist beginning at:\n" + 
			start_name);
  }

  _max_time_check.set_active(false);
  _max_time_entry.set_sensitive(false);

  layout_window();
  setup();
}

ExportDialog::
~ExportDialog() {
}

void ExportDialog::
max_time_toggled() {
  if (_max_time_check.get_active()) {
    _max_time_entry.set_sensitive(true);
  } else {
    _max_time_entry.set_sensitive(false);
  }
}

void ExportDialog::
ok_button_clicked() {
  bool symlink = _symlink_button.get_active();
  int convert_type = -1;
  if (_convert_button.get_active()) {
    convert_type = _convert_type.get_item();
  }

  double max_time = -1;
  if (_max_time_check.get_active()) {
    max_time = atof(_max_time_entry.get_text().c_str());
  }

  new ExportingDialog(_data, _playlist, _start_row,
		      symlink, convert_type,
		      _root_entry.get_filename(),
		      _filename_entry.get_text(),
		      max_time);
  delete_self();
}

void ExportDialog::
cancel_button_clicked() {
  delete_self();
}

void ExportDialog::
layout_window() {
  set_title("Export song files");
  set_default_size(400, 0);

  // The box for the main set of widgets.
  vbox_.pack_start(_box1);
  _box1.show();
  _box1.set_border_width(8);

  // The first explanatory label.
  _box1.pack_start(_top_label, false, false, 8);
  _top_label.show();

  // The radio buttons.
  _box1.pack_start(_rframe, true, true, 8);
  _rframe.show();
  _rframe.set_shadow_type(GTK_SHADOW_ETCHED_IN);
  _rframe.add(_rbox);
  _rbox.show();
  _rbox.set_border_width(8);
  _rbox.pack_start(_symlink_button);
  _symlink_button.show();
  _rbox.pack_start(_copy_button);
  _copy_button.show();
  _rbox.pack_start(_cbox);
  _cbox.show();
  _cbox.pack_start(_convert_button, false, false);
  _convert_button.show();
  _cbox.pack_start(_convert_type);
  _convert_type.show();

  // Root directory, label and entry.
  _root_label.set_alignment(0, 0);
  _box1.pack_start(_root_label, false, false, 0);
  _box1.pack_start(_root_entry);
  _root_label.show();
  _root_entry.show();

  // Filename pattern, label and entry.
  _filename_label.set_alignment(0, 0);
  _box1.pack_start(_filename_label, false, false, 0);
  _box1.pack_start(_filename_entry);
  _filename_label.show();
  _filename_entry.show();
  _filename_explanation.set_justify(GTK_JUSTIFY_LEFT);
  _box1.pack_start(_filename_explanation, false, false, 16);
  _filename_explanation.show();

  // Maximum time, label and entry.
  _box1.pack_start(_max_time_check, false, false, 0);
  _box1.pack_start(_max_time_entry, false, false, 0);
  _max_time_check.show();
  _max_time_entry.show();
  connect_to_method(_max_time_check.toggled, this, &max_time_toggled);

  // The OK button.
  _ok_button.set_usize(80, 36);
  action_area_.pack_start(_ok_button, true, false, 0);
  _ok_button.set_flags(GTK_CAN_DEFAULT);
  _ok_button.grab_default();
  _ok_button.show();
  connect_to_method(_ok_button.clicked, this, &ok_button_clicked);
  
  // The Cancel button.
  _cancel_button.set_usize(80, 36);
  action_area_.pack_start(_cancel_button, true, false, 0);
  _cancel_button.set_flags(GTK_CAN_DEFAULT);
  _cancel_button.show();
  connect_to_method(_cancel_button.clicked, this, &cancel_button_clicked);
}
