/* sfserver.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "daemonProgram.h"
#include "server.h"
#include "playerDef.h"
#include "format.h"

#include <signal.h>
#include <unistd.h>

static const int sfserver_default_port = 8727;
static const string default_sound_device = "/dev/dsp";

class SFServerMain : public DaemonProgram {
public:
  SFServerMain();

  virtual void init();
  virtual void run_daemon();

protected:
  virtual void handle_opts(const string &opt, const string &arg);
  virtual void help();
  virtual void describe_options();

  static void sigterm_handler(int signo);
  static void sighup_handler(int signo);

  string _sound_device;
  int _port;
  static Server *_server;
};

Server *SFServerMain::_server = NULL;
 

SFServerMain::
SFServerMain() : DaemonProgram("sfserver") {
  define_opt("d", true);
  define_opt("p", true);

  _sound_device = default_sound_device;
  _port = sfserver_default_port;
}

void SFServerMain::
init() {
  // First, we try to create a server.  This might fail if it's unable
  // to grab the socket, in which case it will exit() us on the spot.
  _server = new Server(_sound_device, _port);
  
  DaemonProgram::init();
}

void SFServerMain::
run_daemon() {
  // This is formatted as a single string, so it won't be interrupted
  // by output from another process.
  cerr << "sfserver listening on port " + format_int(_port) + "\n";

  // Install a signal handler to catch SIGTERM and SIGINT and anything
  // else likely, so that we can properly close our open sockets on
  // termination.
  struct sigaction act;
  act.sa_handler = sigterm_handler;
  sigemptyset(&act.sa_mask);
  act.sa_flags = 0;
  if (sigaction(SIGTERM, &act, NULL) < 0) {
    perror("sigaction");
    exit(1);
  }
  if (sigaction(SIGINT, &act, NULL) < 0) {
    perror("sigaction");
    exit(1);
  }

  act.sa_handler = sighup_handler;
  if (sigaction(SIGHUP, &act, NULL) < 0) {
    perror("sigaction");
    exit(1);
  }

  // Also ignore SIGPIPE, which we might get from time to time and
  // generally don't care about.
  act.sa_handler = SIG_IGN;
  if (sigaction(SIGPIPE, &act, NULL) < 0) {
    perror("sigaction");
    exit(1);
  }

  PlayerDef::read_config();

  _server->main_loop();

  delete _server;

  exit(0);
}

void SFServerMain::
handle_opts(const string &opt, const string &arg) {
  if (opt == "d") {
    _sound_device = arg;

  } else if (opt == "p") {
    _port = atoi(arg.c_str());

  } else {
    DaemonProgram::handle_opts(opt, arg);
  }
}

void SFServerMain::
help() {
  cerr <<
    "\n"
    "This is a daemon process that assists Paloma in playing sound files and CD\n"
    "tracks.  It should normally be set up to run when the system comes up.\n"

    "sfserver listens on the indicated port for a command from Paloma.  When it\n"
    "receives a command to play a particular sound file, it will run the\n"
    "appropriate helper application to do so, based on the sound file's extension\n"
    "and the information contained in the sfserver configuration file.\n\n"

    "sfserver must connect to a SQL database, but this is only to read the\n"
    "configuration information that specifies which helper programs to execute\n"
    "to actually play the different types of sound files.  Other than this, the\n"
    "sfserver is unrelated to the particular database a Paloma process might be\n"
    "connected to, and a single sfserver can simultaneously serve different Paloma\n"
    "programs that are connected to different databases.\n";

  DaemonProgram::help();
}

void SFServerMain::
describe_options() {
  cerr <<
    "  -d dev     The sound device to access.  The default if this is unspecified\n"
    "             is " << default_sound_device << ".  This may or may not be supported by the various\n"
    "             playback programs that sfserver will contact.\n"
    "  -p port    The port number to listen on for connections.  The default\n"
    "             is " << sfserver_default_port << ".\n";
  DaemonProgram::describe_options();
}

void SFServerMain::
sigterm_handler(int signo) {
  cerr << "Got signal, exiting.\n";
  if (_server == NULL) {
    exit(0);
  } else {
    _server->clean_exit();
  }
}

void SFServerMain::
sighup_handler(int signo) {
  cerr << "Re-reading configuration information.\n";
  PlayerDef::read_config();
}
  
int
main(int argc, char *argv[]) {
  SFServerMain m;
  m.command_line(argc, argv);
  m.init();
  m.run();
  return (0);
}
